#!/usr/bin/env python3
"""
A music streaming application.
"""
import json
import asyncio

import aiohttp
from aiohttp import web, WSMsgType
import jinja2
import aiohttp_jinja2
from aiohttp_jinja2 import render_template
import uvloop
import asyncpg

import config
import events
import database
import buckler_aiohttp

uvloop.install()
routes = web.RouteTableDef()

@routes.get('/', name='index')
async def index(request):
	"""The index page."""
	async with request.app['pool'].acquire() as conn:
		artists = await conn.fetch(
			"SELECT DISTINCT albumartist FROM track ORDER BY albumartist ASC")
	artists = [record['albumartist'] for record in artists]
	return render_template('index.html', request, locals())


@routes.get('/ws', name='ws')
async def websocket_handler(request):
	"""The websocket endpoint."""
	ws = web.WebSocketResponse(heartbeat=30)
	ws_ready = ws.can_prepare(request)
	if not ws_ready.ok:
		return web.Response(text="Cannot start websocket.")
	await ws.prepare(request)

	async for msg in ws:
		if msg.type != WSMsgType.TEXT:
			break

		try:
			data = json.loads(msg.data)
		except json.JSONDecodeError:
			break

		event = data.get('event')
		if not event or event not in events.events.keys():
			break
		await events.events[event](request, ws, data.get('data'))
	await ws.close()
	return ws


async def init_app():
	"""Initializes the application."""
	app = web.Application(middlewares=[buckler_aiohttp.buckler_session])
	aiohttp_jinja2.setup(
		app,
		trim_blocks=True,
		lstrip_blocks=True,
		undefined=jinja2.StrictUndefined,
		loader=jinja2.FileSystemLoader('templates'),
	)
	app['pool'] = await asyncpg.create_pool(**config.db)
	async with app['pool'].acquire() as conn:
		with open('scorch.sql', 'r') as file:
			await conn.execute(file.read())

	app.router.add_routes(routes)

	app_wrap = web.Application()
	app_wrap.add_subapp(config.url_prefix, app)
	return app_wrap


if __name__ == "__main__":
	import sys
	if len(sys.argv) > 1:
		asyncio.run(database.build_library(config.music_dir))
	#app = asyncio.run(init_app())
	#aiohttp.web.run_app(app, host='0.0.0.0', port=5500)
